package BootImage::DependencyResolver;

use strict;
use warnings FATAL => 'all';

use KreaTV;

sub new {
    my ($class, %args) = @_;
    my $self = bless({}, $class);

    $self->{iip_db} = $args{iip_db};
    $self->{version} = $args{version};

    $self->{install} = [];
    $self->{finalize} = [];

    $self->{db} = {};

    return $self;
}

sub _get_iip {
    my ($self, $name) = @_;
    my $iip = $self->{iip_db}->get_iip($name, $self->{version});

    unless ($iip) {
        KreaTV::error("could not find IIP '$name'");
    }

    my $iip_ver = $iip->version();
    if (($iip_ver ne "3") && ($iip_ver ne "2")) {
        KreaTV::error("IIP '$name' has an unsupported version");
    }

    # Use IIP from database if it exists.
    my $iip_name = $iip->info()->name();
    if (exists($self->{db}{$iip_name})) {
        $iip = $self->{db}{$iip_name};
    }

    return $iip;
}

sub _add_iip_to_db {
    my ($self, $iip) = @_;

    # Save IIP in database unless it already exists.
    my $iip_name = $iip->info()->name();
    unless (exists($self->{db}{$iip_name})) {
        $self->{db}{$iip_name} = $iip;
        return 1;
    }

    return 0;
}

sub _check_params {
    my ($self, $iip, $params) = @_;
    my $prev_params = $iip->info()->params();

    if ($iip->version() eq "2") {
        if (defined($params) && $params ne "") {
	    KreaTV::error("IIP " . $iip->info()->name() .
			  " has IIP version 2 which mustn't be " .
			  "used with parameters.");
        }
        return;
    }

    if (defined($params)) {
        if (defined($prev_params) && $params ne $prev_params) {
            KreaTV::error("IIP '" . $iip->info()->name() . "' already " .
                          "installed with different parameters");
        }
	unless (lc($iip->info()->sanity_check()) eq "false") {
	    my @param_names = $iip->info()->param_names();
	    foreach my $param (split(/,/, $params)) {
		next if ($param =~ /^ignoredependency/);
		next if ($param =~ /^prop/);
		unless (grep {$_ eq (split(/=/, $param))[0]} (@param_names)) {
		    KreaTV::error("IIP parameter '" .
				  (split(/=/, $param))[0] .
				  "' is not documented in the IIP " .
				  "description for " . $iip->info()->name());
		}
	    }
	}
        $iip->info()->set_params($params);
    }
}

sub _clear_refcount {
    my ($self, $iip) = @_;

    my $info = $iip->info();
    $info->set_rootdisk_refcount(0);
    $info->set_execution_refcount(0);
    $info->set_finalize_refcount(0);
}

sub _increase_refcount {
    my ($self, $type, $iip) = @_;

    my %dispatch = ( rootdisk  => \&IIP::Info::rootdisk_refcount,
                     execution => \&IIP::Info::execution_refcount,
                     finalize  => \&IIP::Info::finalize_refcount );

    my %set_dispatch = ( rootdisk  => \&IIP::Info::set_rootdisk_refcount,
                         execution => \&IIP::Info::set_execution_refcount,
                         finalize  => \&IIP::Info::set_finalize_refcount );

    my $refcount = $dispatch{$type}->($iip->info()) + 1;
    $set_dispatch{$type}->($iip->info(), $refcount);
    return $refcount;
}

sub _add_dependencies {
    my ($self, $type, $iip) = @_;

    my %get_dependencies = ( rootdisk  => \&IIP::Info::rootdisk_dependencies,
                             execution => \&IIP::Info::execution_dependencies,
                             finalize  => \&IIP::Info::finalize_dependencies );

    my %add_iip = ( rootdisk  => \&add_rootdisk_iip,
                    execution => \&add_execution_iip,
                    finalize  => \&add_finalizer_iip );

    my $dependencies = $get_dependencies{$type}->($iip->info());

    foreach my $dependency (@$dependencies) {
        my ($name, $params) = split(/:/, $dependency, 2);
        $add_iip{$type}->($self, $name, $params);
    }
}

sub _add_iip {
    my ($self, $type, $name, $params) = @_;

    my $iip = $self->_get_iip($name);

    # Check params
    $self->_check_params($iip, $params);

    # Check if already added
    unless ($self->_add_iip_to_db($iip)) {
        if ($self->_increase_refcount($type, $iip) == 1) {
            $self->_add_dependencies($type, $iip);
        }
        return;
    }

    # Add dependencies
    $self->_add_dependencies($type, $iip);

    # Save params
    $iip->info()->set_params($params);

    # Setup refcount
    $self->_clear_refcount($iip);
    $self->_increase_refcount($type, $iip);

    # Add finalizers
    $self->_add_dependencies("finalize", $iip);

    if ($type eq "finalize") {
        push(@{$self->{finalize}}, $iip);
    }
    else {
        push(@{$self->{install}}, $iip);
    }
}

sub add_rootdisk_iip {
    my ($self, $name, $params) = @_;
    $self->_add_iip("rootdisk", $name, $params);
}

sub add_execution_iip {
    my ($self, $name, $params) = @_;
    $self->_add_iip("execution", $name, $params);
}

sub add_finalizer_iip {
    my ($self, $name, $params) = @_;
    $self->_add_iip("finalize", $name, $params);
}

sub add_iips_from_config {
    my ($self, $config) = @_;
    my $iips = $config->iips();
    my $iip_order = $config->iip_order();

    # Fulfil rootdisk and execution dependencies
    foreach my $name (@$iip_order) {
        my $params = $iips->{$name};
        $self->add_rootdisk_iip($name, $params);
        if ($params =~ /ignoredependency\(execution\)/) {
            print "\nIgnored execution dependencies for IIP '$name'\n";
            next;
        }
        $self->add_execution_iip($name, $params);
    }
}

sub install_order {
    my $self = shift;

    my $order = [];
    push(@$order, @{$self->{install}});
    push(@$order, @{$self->{finalize}});
    return $order;
}

1;
