package IIP::Info;

use strict;
use warnings FATAL => 'all';

use KreaTV;

# Elements that will be stored verbatim with attributes and all.
my %verbatim_elements = (Capability => 1);

sub new {
    my $class = shift;
    my $self = bless({}, $class);

    $self->{Name} = undef;
    $self->{Version} = undef;
    $self->{Branch} = undef;
    $self->{Date} = undef;
    $self->{Time} = undef;
    $self->{Architecture} = undef;
    $self->{Capability} = [];
    $self->{ContentVersion} = undef;
    $self->{DependencyOnly} = undef;

    # Dependencies
    $self->{Dependencies} = {};
    $self->{Dependencies}{Rootdisk} = [];
    $self->{Dependencies}{Execution} = [];
    $self->{Dependencies}{Finalize} = [];

    # Description
    $self->{Iipdescription} = undef;
    $self->{Parameters} = {};
    $self->{Sanitycheck} = undef;

    return $self;
}

sub load_from_string {
    my ($hash, $xml) = @_;

    # (.*?) gives the minimal match
    while ($xml =~ /<(\w+)[^>]*>(.*?)<\/\1>/gsi) {
        my $key = $1;
        my $value = $2;

        if (exists $verbatim_elements{$key}) {
            $value = $&; # Save whole XML snippet including attributes.
        }

        if ($key eq "Iip") {
            load_from_string($hash, $value);
        }
        elsif ($key eq "Parameter") {
            my $attrs = {Param => undef, Repeat => undef,
                         Description => undef, Example => undef};
            load_from_string($attrs, $value);
            defined($attrs->{Param})
                or KreaTV::error("Missing Param in Parameter element");
            $hash->{Parameters}{$attrs->{Param}} = $attrs;
        }
        elsif (ref($hash->{$key}) eq "HASH") {
            load_from_string($hash->{$key}, $value);
        }
        elsif (ref($hash->{$key}) eq "ARRAY") {
            push(@{$hash->{$key}}, $value);
        }
        else {
            $hash->{$key} = $value;
        }
    }
}

sub load_from_file {
    my ($self, $file) = @_;
    my $xml = KreaTV::read_file_content($file);
    $self->load_from_string($xml);
}

sub load_from_iip {
    my ($self, $iip) = @_;
    my $xml = KreaTV::backticks("tar zxOf '$iip' info.xml");
    $self->load_from_string($xml);
}

sub _to_xml {
    my ($hash, $prefix) = @_;

    my $xml = "";
    foreach my $key (keys %$hash) {
        if ($key eq "Parameters") {
            foreach my $name (keys %{$hash->{Parameters}}) {
                my $attrs = $hash->{Parameters}{$name};
                $xml .= "$prefix<Parameter>\n";
                foreach (qw(Param Repeat Description Example)) {
                    $xml .= "$prefix  <$_>$attrs->{$_}</$_>\n"
                        if defined($attrs->{$_});
                }
                $xml .= "$prefix</Parameter>\n";
            }
        }
        elsif (ref($hash->{$key}) eq "HASH") {
            my $children = _to_xml($hash->{$key}, $prefix . "  ");
            if ($children) {
                $xml .= $prefix . "<$key>\n";
                $xml .= $children;
                $xml .= $prefix . "</$key>\n";
            }
        }
        elsif (ref($hash->{$key}) eq "ARRAY") {
            foreach my $element (@{$hash->{$key}}) {
                if (exists $verbatim_elements{$key}) {
                    $xml .= $prefix . "$element\n";
                }
                else {
                    $xml .= $prefix . "<$key>$element</$key>\n";
                }
            }
        }
        elsif (defined($hash->{$key})) {
            if (exists $verbatim_elements{$key}) {
                $xml .= $prefix . "$hash->{$key}\n";
            }
            else {
                $xml .= $prefix . "<$key>$hash->{$key}</$key>\n";
            }
        }
    }

    return $xml;
}

sub to_xml {
    my $self = shift;
    return "<?xml version=\"1.0\"?>\n<Iip>\n" . _to_xml($self, "  ") . "</Iip>\n";
}

sub name {
    my $self = shift;
    return $self->{Name};
}

sub set_name {
    my $self = shift;
    $self->{Name} = shift;
}

sub version {
    my $self = shift;
    return $self->{Version};
}

sub set_version {
    my $self = shift;
    $self->{Version} = shift;
}

sub branch {
    my $self = shift;
    return $self->{Branch};
}

sub set_branch {
    my $self = shift;
    $self->{Branch} = shift;
}

sub date {
    my $self = shift;
    return $self->{Date};
}

sub set_date {
    my $self = shift;
    $self->{Date} = shift;
}

sub time {
    my $self = shift;
    return $self->{Time};
}

sub set_time {
    my $self = shift;
    $self->{Time} = shift;
}

sub architecture {
    my $self = shift;
    return $self->{Architecture};
}

sub set_architecture {
    my $self = shift;
    $self->{Architecture} = shift;
}

sub content_version {
    my $self = shift;
    return $self->{ContentVersion};
}

sub set_content_version {
    my $self = shift;
    $self->{ContentVersion} = shift;
}

sub dependency_only {
    my $self = shift;
    return ($self->{DependencyOnly} and $self->{DependencyOnly} eq "true");
}

sub rootdisk_dependencies {
    my $self = shift;
    return $self->{Dependencies}{Rootdisk};
}

sub add_rootdisk_depend {
    my $self = shift;
    push(@{$self->{Dependencies}{Rootdisk}}, shift);
}

sub execution_dependencies {
    my $self = shift;
    return $self->{Dependencies}{Execution};
}

sub add_execution_depend {
    my $self = shift;
    push(@{$self->{Dependencies}{Execution}}, shift);
}

sub finalize_dependencies {
    my $self = shift;
    return $self->{Dependencies}{Finalize};
}

sub add_finalize_depend {
    my $self = shift;
    push(@{$self->{Dependencies}{Finalize}}, shift);
}

sub add_capability {
    my $self = shift;
    my $type = shift;
    my $cap = shift;
    my $s;
    if ($type) {
        $s = "<Capability type=\"$type\">$cap</Capability>";
    }
    else {
        $s = "<Capability>$cap</Capability>";
    }
    push(@{$self->{Capability}}, $s);
}

sub full_name {
    my $self = shift;
    return
        $self->name .
        "_" . $self->version .
        ($self->architecture ne "" ? "_" . $self->architecture : "");
}

sub processor {
    my $self = shift;
    return (split(/_/, $self->architecture))[0];
}

sub bi_compat {
    my $self = shift;
    return (split(/_/, $self->architecture))[1];
}

sub dbl_compat {
    my $self = shift;
    return (split(/_/, $self->architecture))[2];
}

sub rootdisk_refcount {
    my $self = shift;
    return $self->{RootdiskRefCount} || 0;
}

sub set_rootdisk_refcount {
    my $self = shift;
    $self->{RootdiskRefCount} = shift;
}

sub execution_refcount {
    my $self = shift;
    return $self->{ExecutionRefCount} || 0;
}

sub set_execution_refcount {
    my $self = shift;
    $self->{ExecutionRefCount} = shift;
}

sub finalize_refcount {
    my $self = shift;
    return $self->{FinalizeRefCount} || 0;
}

sub set_finalize_refcount {
    my $self = shift;
    $self->{FinalizeRefCount} = shift;
}

sub description {
    my $self = shift;
    return $self->{Iipdescription};
}

sub set_description {
    my $self = shift;
    $self->{Iipdescription} = shift;
}

sub param_names {
    my $self = shift;
    return keys %{$self->{Parameters}};
}

sub param_repeat {
    my ($self, $name) = @_;
    my $val = $self->{Parameters}{$name}{Repeat};
    return defined($val) ? $val : "";
}

sub param_description {
    my ($self, $name) = @_;
    my $val = $self->{Parameters}{$name}{Description};
    return defined($val) ? $val : "";
}

sub param_example {
    my ($self, $name) = @_;
    my $val = $self->{Parameters}{$name}{Example};
    return defined($val) ? $val : "";
}

sub add_parameter {
    my ($self, $name, $repeat, $description, $example) = @_;
    $self->{Parameters}{$name}{Param} = $name;
    $self->{Parameters}{$name}{Repeat} = $repeat;
    $self->{Parameters}{$name}{Description} = $description;
    $self->{Parameters}{$name}{Example} = $example;
}

sub sanity_check {
    my $self = shift;
    return lc($self->{Sanitycheck}) if defined $self->{Sanitycheck};
    return "true";
}

sub params {
    my $self = shift;
    return $self->{Params};
}

sub set_params {
    my $self = shift;
    $self->{Params} = shift;
}

1;
