#!/bin/bash

# Build DBL update IIP for Motorola IP-STBs
#
# This script is meant to be run in a KBK environment.
#
# (c) Motorola Inc 2008


### Paths

# These may have to be modified to correspond to your installation
# The default values correspond to that the Application Suite and
# development kit are both unpacked in the same directory


# DK should point to the directory where your development kit (xDK)
# is unpacked
DK=$(dirname $0)/..

# This points to the location where your resulting IIP is stored
IIPS=$(pwd)/iip/


### Check parameter

DBL=$1
STBCONFIG=$2

if [ -z $DBL ]; then
  echo "Usage: build_dbl_iip.sh <DBL> <config file>"
  echo
  echo "The DBL must be named according to this:"
  echo "kreatv-dbl_<version number>_<IP-STB model>_<boot-image compatibility>_<DBL compatibility>.bin"
  echo "Example: kreatv-dbl_1.36_st40_vip19x0.bin"
  echo
  echo "The config file is used e.g. on the ST40 and ends up as"
  echo "/flash/.config_s1 or /flash/.config_s2 depending on which DBL slot"
  echo "is currently being used. Including the config file in the IIP is"
  echo "optional."
  echo
  echo -n "The resulting IIP is created in the output directory"
  echo " ("$IIPS")"
  exit 0
fi

DBL_FILE=$(basename $DBL)
DBL_VERSION=$(echo $DBL_FILE | cut -d _ -f 2)
DBL_ARCH=$(echo $DBL_FILE | cut -d _ -f 3-6 | cut -d . -f 1)


### Convert from old format
DBL_ARCH=$(echo $DBL_ARCH | sed "s/sh4-kreatel-linux-gnu/st40/")
DBL_CHIP=$(echo $DBL_ARCH | cut -d _ -f 1)
DBL_PROC=$(echo $DBL_ARCH | cut -d _ -f 2)

if [ "$DBL_PROC" = "vip19x0" ]; then
  DBL_BICOMP=25
  DBL_FWCOMP=25
fi

if [ "$DBL_PROC" = "vip19x3" ]; then
  DBL_BICOMP=29
  DBL_FWCOMP=29
fi

if [ "$DBL_PROC" = "vip20x0" ]; then
  DBL_BICOMP=30
  DBL_FWCOMP=30
fi

if [ "$DBL_PROC" = "vip22x2" ]; then
  DBL_BICOMP=31
  DBL_FWCOMP=31
fi

if [ "$DBL_PROC" = "vip26x2" ]; then
  DBL_BICOMP=32
  DBL_FWCOMP=32
fi

if [ "$DBL_PROC" = "vip10x2" ]; then
  DBL_BICOMP=33
  DBL_FWCOMP=33
fi

if [ -z $DBL_BICOMP ]; then
    echo "Error: Missing boot-image compatibility in filename"
    echo "Example: kreatv-dbl_1.36_st40_vip19x0.bin"
    echo "         This number is missing --^"
    exit 1
fi
if [ -z $DBL_FWCOMP ]; then
    echo "Error: Missing DBL compatibility in filename"
    echo "Example: kreatv-dbl_1.36_st40_vip19x0.bin"
    echo "         This number is missing -----^"
    exit 1
fi

echo " Content version: $DBL_VERSION"
echo " Architecture: $DBL_ARCH"
echo "\"Processor\": $DBL_PROC"
echo " Boot image compatibility: $DBL_BICOMP"
echo " Firmware compatibility: $DBL_FWCOMP"


### Check if development kit fits this architecture

if [ ! -d ${DK}/$DBL_CHIP ]; then
    echo ""
    echo "Error: Cannot generate the requested IIP; this development kit does not match architecture \"$DBL_CHIP\""
    echo ""
    echo "Make sure your DBL is named according to this:"
    echo "kreatv-dbl_<version number>_<IP-STB model>_<boot-image compatibility>_<DBL compatibility>.bin"
    echo "Example: kreatv-dbl_1.36_st40_vip19x0_29_29.bin"
    echo "If the DBL seem to have a valid name and it still does not work, you might"
    echo "need to download a development kit supporting architecture \"$DBL_PROC\""
    exit 1
fi


### Check existence of output directory

if [ ! -d $IIPS ] ;
then
  echo 'The directory '$IIPS' does not exist, creating.'
  mkdir -p $IIPS
fi


### Get version

if [ -d ${DK}/kbk ]; then
  XMLDIR=${DK}/kbk;
  VERSION=$(cat $XMLDIR/build_data.xml | grep Version | cut -f 2 -d  '>' | cut -f 1 -d '<')
else
  echo "Could not find build_data.xml: Skipping setting boot-image version (which is ok)"
fi


### Build DBL update IIP

IIP_NAME=kreatv-update-dbl-${DBL_VERSION}_${VERSION}_${DBL_ARCH}.iip
IIP_INSTALL=${DK}/bin/update-dbl.sh


### Build data dir
TMP_DIR=$(mktemp -d /tmp/iipdata.XXXXXX)
DATA_DIR=${TMP_DIR}/data
mkdir -p ${DATA_DIR}
cp ${DBL} ${DATA_DIR}/dbl.bin

if [ "$STBCONFIG" != "" ]; then
    if [ -e ${STBCONFIG} ]; then
        echo "Config file: ${STBCONFIG}"
        cp ${STBCONFIG} ${DATA_DIR}/config.xml
    else
        echo "Error: Cannot find config file \"${STBCONFIG}\""
        exit 1
    fi
fi

cp ${DK}/$DBL_CHIP/bin/firmwaremanager ${DATA_DIR} || exit 1


${DK}/bin/iip_build \
  --name kreatv-update-dbl-${DBL_VERSION} \
  --version ${VERSION} \
  --date $(date +%y%m%d) \
  --content_version ${DBL_VERSION} \
  --time $(date +%H%M%S) \
  --processor ${DBL_PROC} \
  --hw_comp ${DBL_BICOMP} \
  --fw_comp ${DBL_FWCOMP} \
  --depends rootdisk:kreatv-platform \
  --depends execution:kreatv-option-toish \
  --depends execution:kreatv-option-softwareservice \
  --install runtime:${IIP_INSTALL} \
  --data_dir ${DATA_DIR} \
  --out_dir ${IIPS}
if [ $? != 0 ]; then
  echo "Error: IIP build failed"
  STATUS=1
else
  echo
  echo "Done"
  echo "The IIP is here: $IIPS$IIP_NAME"
  STATUS=0
fi

rm -rf ${TMP_DIR}

exit $STATUS
